@@ -43,7 +43,18 @@ from supybot.i18n import PluginInternationalization, internationalizeDocstring
 _ = PluginInternationalization('Relay')
 
 class Relay(callbacks.Plugin):
-    """This plugin allows you to setup a relay between networks."""
+    """
+    This plugin allows you to setup a relay between networks.
+
+    Note that you must tell the bot to join the channel you wish to relay on
+    all networks with the ``join`` command or
+    ``network command <network> join <channel>``
+    or to join the channel on all networks ``network cmdall join <channel>``.
+
+    There are several advanced alternatives to this plugin, available as
+    third-party plugins. You can check them out at
+    https://limnoria.net/plugins.xhtml#messaging
+    """
     noIgnore = True
     def __init__(self, irc):
         self.__parent = super(Relay, self)
